
package com.sreal.visiprox;

public class VisiProx
{
	private class PumpThread extends java.lang.Thread
	{
		public static final int DIRECTION_CLIENT_TO_SERVER = 0;
		public static final int DIRECTION_SERVER_TO_CLIENT = 1;

		private com.sreal.visiprox.VisiProx parent;
		private int direction;

		public PumpThread(com.sreal.visiprox.VisiProx parent, int direction)
		{
			this.parent = parent;
			this.direction = direction;
		}

		public void run()
		{
			java.io.InputStream input_stream;
			java.io.OutputStream output_stream;
			javax.swing.JTextArea display;
			javax.swing.BoundedRangeModel display_scroll_model;
			byte[] buffer = new byte[4096];
			int amount;

			try
			{
				if (this.direction == DIRECTION_CLIENT_TO_SERVER)
				{
					input_stream = this.parent.socket_to_client.getInputStream();
					output_stream = this.parent.socket_to_server.getOutputStream();
					display = this.parent.client_to_server_display;
					display_scroll_model = this.parent.client_to_server_display_scroll_model;
				}
				else
				{
					input_stream = this.parent.socket_to_server.getInputStream();
					output_stream = this.parent.socket_to_client.getOutputStream();
					display = this.parent.server_to_client_display;
					display_scroll_model = this.parent.server_to_client_display_scroll_model;
				}

				while (true)
				{
					int display_format;

					amount = input_stream.read(buffer);

					if (amount == -1)
					{
						break;
					}

					if (this.direction == DIRECTION_CLIENT_TO_SERVER)
					{
						display_format = this.parent.client_to_server_display_format;
					}
					else
					{
						display_format = this.parent.server_to_client_display_format;
					}

					switch (display_format)
					{
						case com.sreal.visiprox.VisiProx.DISPLAY_FORMAT_TEXT:
						{
							display.append(new java.lang.String(buffer, 0, amount));
							break;
						}
						case com.sreal.visiprox.VisiProx.DISPLAY_FORMAT_BINARY:
						{
							int i;
							java.lang.StringBuffer scratch_buffer = new java.lang.StringBuffer(amount + 3);

							for (i = 0; i < amount; i++)
							{
								scratch_buffer.append(parent.format_converter.byteToHexString(buffer[i]));
							}

							display.append(scratch_buffer.toString());

							break;
						}
						case com.sreal.visiprox.VisiProx.DISPLAY_FORMAT_MIXED:
						{
							int i;
							java.lang.StringBuffer scratch_buffer = new java.lang.StringBuffer(amount + 3);

							for (i = 0; i < amount; i++)
							{
								scratch_buffer.append(parent.format_converter.byteToMixedString(buffer[i]));
							}

							display.append(scratch_buffer.toString());

							break;
						}
					}

					display_scroll_model.setValue(display_scroll_model.getMaximum());

					output_stream.write(buffer, 0, amount);
				}
			}
			catch (java.io.IOException exception)
			{
			}

			if (this.direction == DIRECTION_CLIENT_TO_SERVER)
			{
				this.parent.notifyClientToServerPumpIsFinished(true);
			}
			else
			{
				this.parent.notifyServerToClientPumpIsFinished(true);
			}
		}
	}

	private final static int DISPLAY_FORMAT_TEXT = 0;
	private final static int DISPLAY_FORMAT_BINARY = 1;
	private final static int DISPLAY_FORMAT_MIXED = 2;

	FormatConverter format_converter = new com.sreal.visiprox.FormatConverter();
	private java.lang.String proxy_hostname;
	private int proxy_port;
	private java.lang.String server_hostname;
	private int server_port;
	private boolean client_to_server_pump_is_finished;
	private boolean server_to_client_pump_is_finished;
	java.net.Socket socket_to_client;
	java.net.Socket socket_to_server;
	javax.swing.JFrame frame;
	javax.swing.JTextArea client_to_server_display;
	javax.swing.JTextArea server_to_client_display;
	javax.swing.BoundedRangeModel client_to_server_display_scroll_model;
	javax.swing.BoundedRangeModel server_to_client_display_scroll_model;
	javax.swing.JLabel client_label;
	int client_to_server_display_format = DISPLAY_FORMAT_TEXT;
	int server_to_client_display_format = DISPLAY_FORMAT_TEXT;
	boolean clear_on_connect = true;

	public VisiProx(int proxy_port, java.lang.String server_hostname, int server_port)
	{
		this.proxy_port = proxy_port;
		this.server_hostname = server_hostname;
		this.server_port = server_port;

		try
		{
			this.proxy_hostname = java.net.InetAddress.getLocalHost().getHostName();
		}
		catch (java.net.UnknownHostException exception)
		{
			this.proxy_hostname = "localhost";
		}

		this.createGui();
		this.listenForConnections();
	}

	public static void main(java.lang.String[] args)
	{
		int proxy_port;
		java.lang.String server_hostname;
		int server_port;
		com.sreal.visiprox.VisiProx visiprox;

		setGuiLookAndFeel();

		try
		{
			proxy_port = java.lang.Integer.parseInt(args[0]);
			server_hostname = args[1];
			server_port = java.lang.Integer.parseInt(args[2]);

			visiprox = new com.sreal.visiprox.VisiProx(proxy_port, server_hostname, server_port);
		}
		catch (java.lang.NumberFormatException exception)
		{
			javax.swing.JOptionPane.showMessageDialog(null, "Usage: java com.sreal.visiprox.VisiProx <proxy port> <server hostname> <server port>", "Usage", javax.swing.JOptionPane.INFORMATION_MESSAGE);
			java.lang.System.exit(1);
		}
		catch (java.lang.ArrayIndexOutOfBoundsException exception)
		{
			javax.swing.JOptionPane.showMessageDialog(null, "Usage: java com.sreal.visiprox.VisiProx <proxy port> <server hostname> <server port>", "Usage", javax.swing.JOptionPane.INFORMATION_MESSAGE);
			java.lang.System.exit(1);
		}
	}

	public static void setGuiLookAndFeel()
	{
		javax.swing.plaf.metal.DefaultMetalTheme theme;
		javax.swing.plaf.metal.MetalLookAndFeel plaf;

		theme = new javax.swing.plaf.metal.DefaultMetalTheme()
		{
			private javax.swing.plaf.ColorUIResource primary1 = new javax.swing.plaf.ColorUIResource(java.awt.Color.getColor("com.sreal.visiprox.primary1", new java.awt.Color(0x666666)));
			private javax.swing.plaf.ColorUIResource primary2 = new javax.swing.plaf.ColorUIResource(java.awt.Color.getColor("com.sreal.visiprox.primary2", new java.awt.Color(0xFF9966)));
			private javax.swing.plaf.ColorUIResource primary3 = new javax.swing.plaf.ColorUIResource(java.awt.Color.getColor("com.sreal.visiprox.primary3", new java.awt.Color(0xFFCC99)));
			private javax.swing.plaf.ColorUIResource secondary1 = new javax.swing.plaf.ColorUIResource(java.awt.Color.getColor("com.sreal.visiprox.secondary1", new java.awt.Color(0x666666)));
			private javax.swing.plaf.ColorUIResource secondary2 = new javax.swing.plaf.ColorUIResource(java.awt.Color.getColor("com.sreal.visiprox.secondary2", new java.awt.Color(0x999999)));
			private javax.swing.plaf.ColorUIResource secondary3 = new javax.swing.plaf.ColorUIResource(java.awt.Color.getColor("com.sreal.visiprox.secondary3", new java.awt.Color(0xCCCCCC)));
			private javax.swing.plaf.FontUIResource control_text_font = new javax.swing.plaf.FontUIResource(java.awt.Font.getFont("com.sreal.visiprox.control_text_font", new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12)));
			private javax.swing.plaf.FontUIResource menu_text_font = new javax.swing.plaf.FontUIResource(java.awt.Font.getFont("com.sreal.visiprox.menu_text_font", new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12)));
			private javax.swing.plaf.FontUIResource sub_text_font = new javax.swing.plaf.FontUIResource(java.awt.Font.getFont("com.sreal.visiprox.sub_text_font", new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12)));
			private javax.swing.plaf.FontUIResource system_text_font = new javax.swing.plaf.FontUIResource(java.awt.Font.getFont("com.sreal.visiprox.system_text_font", new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12)));
			private javax.swing.plaf.FontUIResource user_text_font = new javax.swing.plaf.FontUIResource(java.awt.Font.getFont("com.sreal.visiprox.user_text_font", new java.awt.Font("Monospaced", java.awt.Font.PLAIN, 12)));
			private javax.swing.plaf.FontUIResource window_title_font = new javax.swing.plaf.FontUIResource(java.awt.Font.getFont("com.sreal.visiprox.window_title_font", new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12)));

			public javax.swing.plaf.ColorUIResource getPrimary1()
			{
				return this.primary1;
			}

			public javax.swing.plaf.ColorUIResource getPrimary2()
			{
				return this.primary2;
			}

			public javax.swing.plaf.ColorUIResource getPrimary3()
			{
				return this.primary3;
			}

			public javax.swing.plaf.ColorUIResource getSecondary1()
			{
				return this.secondary1;
			}

			public javax.swing.plaf.ColorUIResource getSecondary2()
			{
				return this.secondary2;
			}

			public javax.swing.plaf.ColorUIResource getSecondary3()
			{
				return this.secondary3;
			}

			public javax.swing.plaf.FontUIResource getControlTextFont()
			{
				return this.control_text_font;
			}

			public javax.swing.plaf.FontUIResource getMenuTextFont()
			{
				return this.menu_text_font;
			}

			public javax.swing.plaf.FontUIResource getSubTextFont()
			{
				return this.sub_text_font;
			}

			public javax.swing.plaf.FontUIResource getSystemTextFont()
			{
				return this.system_text_font;
			}

			public javax.swing.plaf.FontUIResource getUserTextFont()
			{
				return this.user_text_font;
			}

			public javax.swing.plaf.FontUIResource getWindowTitleFont()
			{
				return this.window_title_font;
			}
		};

		plaf = new javax.swing.plaf.metal.MetalLookAndFeel();
		plaf.setCurrentTheme(theme);

		try
		{
			javax.swing.UIManager.setLookAndFeel(plaf);
		}
		catch (javax.swing.UnsupportedLookAndFeelException exception)
		{
		}
	}

	private void createGui()
	{
		javax.swing.JPanel panel;
		java.awt.GridBagLayout layout;
		javax.swing.JSplitPane split_pane;
		javax.swing.JScrollPane scroll_pane;
		java.awt.GridBagConstraints constraints;
		javax.swing.JPanel status_panel;
		java.awt.GridLayout status_layout;
		javax.swing.JLabel label;
		javax.swing.JMenuBar menu_bar;
		javax.swing.JMenu menu;
		javax.swing.JMenu submenu;
		javax.swing.JMenuItem menu_item;
		javax.swing.JCheckBoxMenuItem check_box_menu_item;
		javax.swing.JRadioButtonMenuItem radio_button_menu_item;
		javax.swing.ButtonGroup button_group;
		java.awt.event.ActionListener action_listener;
		java.awt.event.ItemListener item_listener;
		java.awt.event.WindowListener window_listener;
		java.awt.event.ComponentListener component_listener;

		frame = new javax.swing.JFrame("VisiProx");

		panel = new javax.swing.JPanel();
		frame.setContentPane(panel);

		layout = new java.awt.GridBagLayout();
		panel.setLayout(layout);

		split_pane = new javax.swing.JSplitPane(javax.swing.JSplitPane.VERTICAL_SPLIT);
		split_pane.setDividerSize(4);
		panel.add(split_pane);

		constraints = new java.awt.GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, java.awt.GridBagConstraints.CENTER, java.awt.GridBagConstraints.BOTH, new java.awt.Insets(4, 4, 2, 4), 0, 0);
		layout.setConstraints(split_pane, constraints);

		component_listener = new java.awt.event.ComponentAdapter()
		{
			public void componentResized(java.awt.event.ComponentEvent event)
			{
				javax.swing.JSplitPane target;

				target = (javax.swing.JSplitPane)(event.getComponent());
				target.setDividerLocation(0.5);
			}
		};
		split_pane.addComponentListener(component_listener);

		this.client_to_server_display = new javax.swing.JTextArea();
		this.client_to_server_display.setEditable(false);
		this.client_to_server_display.setLineWrap(true);

		scroll_pane = new javax.swing.JScrollPane(this.client_to_server_display, javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		this.client_to_server_display_scroll_model = scroll_pane.getVerticalScrollBar().getModel();
		split_pane.setTopComponent(scroll_pane);

		this.server_to_client_display = new javax.swing.JTextArea();
		this.server_to_client_display.setEditable(false);
		this.server_to_client_display.setLineWrap(true);

		scroll_pane = new javax.swing.JScrollPane(this.server_to_client_display, javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		this.server_to_client_display_scroll_model = scroll_pane.getVerticalScrollBar().getModel();
		split_pane.setBottomComponent(scroll_pane);

		status_panel = new javax.swing.JPanel();
		panel.add(status_panel);

		constraints = new java.awt.GridBagConstraints(0, 1, 1, 1, 1.0, 0.0, java.awt.GridBagConstraints.CENTER, java.awt.GridBagConstraints.HORIZONTAL, new java.awt.Insets(2, 4, 4, 4), 0, 0);
		layout.setConstraints(status_panel, constraints);

		status_layout = new java.awt.GridLayout(1, 3, 0, 0);
		status_panel.setLayout(status_layout);

		this.client_label = new javax.swing.JLabel("(not connected)");
		this.client_label.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createEmptyBorder(2, 2, 2, 2)));
		status_panel.add(this.client_label);

		label = new javax.swing.JLabel(this.proxy_hostname + ":" + this.proxy_port);
		label.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEmptyBorder(0, 4, 0, 4), javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createEmptyBorder(2, 2, 2, 2))));
		status_panel.add(label);

		label = new javax.swing.JLabel(this.server_hostname + ":" + this.server_port);
		label.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEtchedBorder(), javax.swing.BorderFactory.createEmptyBorder(2, 2, 2, 2)));
		status_panel.add(label);

		menu_bar = new javax.swing.JMenuBar();
		frame.setJMenuBar(menu_bar);

		menu = new javax.swing.JMenu("File");
		menu.setMnemonic(java.awt.event.KeyEvent.VK_F);
		menu_bar.add(menu);

		menu_item = new javax.swing.JMenuItem("Save client to server");
		menu_item.setMnemonic(java.awt.event.KeyEvent.VK_C);
		menu.add(menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				javax.swing.JFileChooser file_chooser;
				java.io.File file;
				java.io.FileWriter writer;

				try
				{
					file_chooser = new javax.swing.JFileChooser();
					file_chooser.showSaveDialog(frame);
					file = file_chooser.getSelectedFile();
					writer = new java.io.FileWriter(file);
					writer.write(client_to_server_display.getText());
					writer.close();
				}
				catch (java.lang.Exception exception)
				{
					javax.swing.JOptionPane.showMessageDialog(null, "Could not write file.", "Error", javax.swing.JOptionPane.ERROR_MESSAGE);
				}
			}
		};
		menu_item.addActionListener(action_listener);

		menu_item = new javax.swing.JMenuItem("Save server to client");
		menu_item.setMnemonic(java.awt.event.KeyEvent.VK_S);
		menu.add(menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				javax.swing.JFileChooser file_chooser;
				java.io.File file;
				java.io.FileWriter writer;

				try
				{
					file_chooser = new javax.swing.JFileChooser();
					file_chooser.showSaveDialog(frame);
					file = file_chooser.getSelectedFile();
					writer = new java.io.FileWriter(file);
					writer.write(server_to_client_display.getText());
					writer.close();
				}
				catch (java.lang.Exception exception)
				{
					javax.swing.JOptionPane.showMessageDialog(null, "Could not write file.", "Error", javax.swing.JOptionPane.ERROR_MESSAGE);
				}
			}
		};
		menu_item.addActionListener(action_listener);

		menu.addSeparator();

		menu_item = new javax.swing.JMenuItem("Exit");
		menu_item.setMnemonic(java.awt.event.KeyEvent.VK_X);
		menu.add(menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				java.lang.System.exit(0);
			}
		};
		menu_item.addActionListener(action_listener);

		menu = new javax.swing.JMenu("View");
		menu.setMnemonic(java.awt.event.KeyEvent.VK_V);
		menu_bar.add(menu);

		submenu = new javax.swing.JMenu("Client to server format");
		submenu.setMnemonic(java.awt.event.KeyEvent.VK_C);
		menu.add(submenu);

		button_group = new javax.swing.ButtonGroup();

		radio_button_menu_item = new javax.swing.JRadioButtonMenuItem("Text");
		radio_button_menu_item.setMnemonic(java.awt.event.KeyEvent.VK_T);
		button_group.add(radio_button_menu_item);
		radio_button_menu_item.setSelected(true);
		submenu.add(radio_button_menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				client_to_server_display_format = DISPLAY_FORMAT_TEXT;
				client_to_server_display.setWrapStyleWord(false);
			}
		};
		radio_button_menu_item.addActionListener(action_listener);

		radio_button_menu_item = new javax.swing.JRadioButtonMenuItem("Binary");
		radio_button_menu_item.setMnemonic(java.awt.event.KeyEvent.VK_B);
		button_group.add(radio_button_menu_item);
		submenu.add(radio_button_menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				client_to_server_display_format = DISPLAY_FORMAT_BINARY;
				client_to_server_display.setWrapStyleWord(true);
			}
		};
		radio_button_menu_item.addActionListener(action_listener);

		radio_button_menu_item = new javax.swing.JRadioButtonMenuItem("Mixed");
		radio_button_menu_item.setMnemonic(java.awt.event.KeyEvent.VK_M);
		button_group.add(radio_button_menu_item);
		submenu.add(radio_button_menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				client_to_server_display_format = DISPLAY_FORMAT_MIXED;
				client_to_server_display.setWrapStyleWord(true);
			}
		};
		radio_button_menu_item.addActionListener(action_listener);

		submenu = new javax.swing.JMenu("Server to client format");
		submenu.setMnemonic(java.awt.event.KeyEvent.VK_S);
		menu.add(submenu);

		button_group = new javax.swing.ButtonGroup();

		radio_button_menu_item = new javax.swing.JRadioButtonMenuItem("Text");
		radio_button_menu_item.setMnemonic(java.awt.event.KeyEvent.VK_T);
		button_group.add(radio_button_menu_item);
		radio_button_menu_item.setSelected(true);
		submenu.add(radio_button_menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				server_to_client_display_format = DISPLAY_FORMAT_TEXT;
				server_to_client_display.setWrapStyleWord(false);
			}
		};
		radio_button_menu_item.addActionListener(action_listener);

		radio_button_menu_item = new javax.swing.JRadioButtonMenuItem("Binary");
		radio_button_menu_item.setMnemonic(java.awt.event.KeyEvent.VK_B);
		button_group.add(radio_button_menu_item);
		submenu.add(radio_button_menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				server_to_client_display_format = DISPLAY_FORMAT_BINARY;
				server_to_client_display.setWrapStyleWord(true);
			}
		};
		radio_button_menu_item.addActionListener(action_listener);

		radio_button_menu_item = new javax.swing.JRadioButtonMenuItem("Mixed");
		radio_button_menu_item.setMnemonic(java.awt.event.KeyEvent.VK_M);
		button_group.add(radio_button_menu_item);
		submenu.add(radio_button_menu_item);

		action_listener = new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent event)
			{
				server_to_client_display_format = DISPLAY_FORMAT_MIXED;
				server_to_client_display.setWrapStyleWord(true);
			}
		};
		radio_button_menu_item.addActionListener(action_listener);

		menu.addSeparator();

		check_box_menu_item = new javax.swing.JCheckBoxMenuItem("Clear on connect");
		check_box_menu_item.setMnemonic(java.awt.event.KeyEvent.VK_L);
		check_box_menu_item.setSelected(true);
		menu.add(check_box_menu_item);

		item_listener = new java.awt.event.ItemListener()
		{
			public void itemStateChanged(java.awt.event.ItemEvent event)
			{
				clear_on_connect = (event.getStateChange() == java.awt.event.ItemEvent.SELECTED);
			}
		};
		check_box_menu_item.addItemListener(item_listener);

		window_listener = new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent event)
			{
				java.lang.System.exit(0);
			}
		};
		frame.addWindowListener(window_listener);

		component_listener = new java.awt.event.ComponentAdapter()
		{
			public void componentShown(java.awt.event.ComponentEvent event)
			{
				java.awt.Component component;

				component = event.getComponent();
				component.repaint();
			}

			public void componentResized(java.awt.event.ComponentEvent event)
			{
				java.awt.Component component;

				component = event.getComponent();
				component.repaint();
			}
		};
		frame.addComponentListener(component_listener);

		frame.setBounds(0, 0, 0, 0);
		frame.setVisible(true);
		frame.setBounds(25, 25, 640, 480);
	}

	private void listenForConnections()
	{
		java.net.ServerSocket listen_socket;
		java.lang.String client_hostname;
		int client_port;
		PumpThread client_to_server_pump;
		PumpThread server_to_client_pump;

		try
		{
			listen_socket = new java.net.ServerSocket(proxy_port);

			while (true)
			{
				this.socket_to_client = listen_socket.accept();
				this.socket_to_server = new java.net.Socket(this.server_hostname, this.server_port);

				client_hostname = this.socket_to_client.getInetAddress().getHostName();
				client_port = this.socket_to_client.getPort();
				this.client_label.setText(client_hostname + ":" + client_port);

				if (this.clear_on_connect)
				{
					this.client_to_server_display.setText("");
					this.server_to_client_display.setText("");
				}

				client_to_server_pump = new PumpThread(this, PumpThread.DIRECTION_CLIENT_TO_SERVER);
				server_to_client_pump = new PumpThread(this, PumpThread.DIRECTION_SERVER_TO_CLIENT);

				this.notifyClientToServerPumpIsFinished(false);
				this.notifyServerToClientPumpIsFinished(false);

				client_to_server_pump.start();
				server_to_client_pump.start();

				this.waitForPumpsToFinish();

				this.socket_to_client.close();
				this.socket_to_server.close();

				this.client_label.setText("(not connected)");
			}
		}
		catch (java.io.IOException exception)
		{
			javax.swing.JOptionPane.showMessageDialog(null, exception.getMessage(), "Error", javax.swing.JOptionPane.ERROR_MESSAGE);
			java.lang.System.exit(1);
		}
	}

	synchronized void notifyClientToServerPumpIsFinished(boolean value)
	{
		this.client_to_server_pump_is_finished = value;
		this.notifyAll();
	}

	synchronized void notifyServerToClientPumpIsFinished(boolean value)
	{
		this.server_to_client_pump_is_finished = value;
		this.notifyAll();
	}

	private synchronized void waitForPumpsToFinish()
	{
		while (!this.client_to_server_pump_is_finished && !this.server_to_client_pump_is_finished)
		{
			try
			{
				this.wait();
			}
			catch (java.lang.InterruptedException exception)
			{
			}
		}
	}
}

